// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"github.com/stretchr/testify/assert"
	"path/filepath"
	"testing"
)

// Example for README
// Print are commented out because they are only for demonstration
func TestReadExample3(t *testing.T) {

	config := struct {
		Host string
		Flag bool
	}{
		Host: "localhost",
		Flag: true,
	}

	c := New(config)

	c.SetMnemonic("my-app")
	c.SetDefaultDirectories()
	c.Import()

	assert.Equal(t, c.Config().Host, "localhost")

}

type TestStruct struct {
	Path1 PathValue
	Path2 string
	Path3 []PathValue
}

func TestReplacePath2(t *testing.T) {
	testData := TestStruct{
		Path1: "relative/path1",
		Path2: "relative/path2",
		Path3: []PathValue{"relative/path3", "relative/path4"},
	}
	basePath := "/base"

	err := replacePath[*TestStruct](basePath, &testData)
	assert.Nil(t, err)

	expectedPath1 := PathValue(filepath.Join(basePath, "relative/path1"))
	if testData.Path1 != expectedPath1 {
		t.Errorf("Expected %s, got %s", expectedPath1, testData.Path1)
	}

	// no pathValue
	expectedPath2 := "relative/path2"
	if testData.Path2 != expectedPath2 {
		t.Errorf("Expected %s, got %s", expectedPath2, testData.Path2)
	}

	expectedPath3 := []PathValue{PathValue(filepath.Join(basePath, "relative/path3")), PathValue(filepath.Join(basePath, "relative/path4"))}
	if testData.Path3[0] != expectedPath3[0] {
		t.Errorf("Expected %s, got %s", expectedPath3[0], testData.Path3[0])
	}

	if testData.Path3[1] != expectedPath3[1] {
		t.Errorf("Expected %s, got %s", expectedPath3[1], testData.Path3[1])
	}
}

type SubTestSubPaths struct {
	Template    PathValue
	Definitions []PathValue
}

type SubTest2Def struct {
	Paths SubTestSubPaths
}

type SubTestStruct1 map[string]SubTest2Def
type MainTestStruct struct {
	Sub SubTestStruct1
}

func TestReplacePathForConfig(t *testing.T) {
	config := MainTestStruct{
		Sub: SubTestStruct1{
			"Default": SubTest2Def{
				Paths: SubTestSubPaths{
					Template:    "../../../default.html",
					Definitions: []PathValue{"../../../legacy.yaml"},
				},
			},
		},
	}

	basePath := "/base/1/2/3/"

	err := replacePath[*MainTestStruct](basePath, &config)
	if err != nil {
		t.Error(err)
	}

	expectedTemplatePath := PathValue(filepath.Join(basePath, "../../../default.html"))
	if config.Sub["Default"].Paths.Template != expectedTemplatePath {
		t.Errorf("Expected %s, got %s", expectedTemplatePath, config.Sub["Default"].Paths.Template)
	}

	expectedDefinitionPath := PathValue(filepath.Join(basePath, "../../../legacy.yaml"))
	if config.Sub["Default"].Paths.Definitions[0] != expectedDefinitionPath {
		t.Errorf("Expected %s, got %s", expectedDefinitionPath, config.Sub["Default"].Paths.Definitions[0])
	}
}
